/**
 * Copyright (c) 2002-2015 "Neo Technology,"
 * Network Engine for Objects in Lund AB [http://neotechnology.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.kernel.monitoring;

import java.io.File;

public interface StoreCopyMonitor
{

    void startCopyingFiles();

    void finishedCopyingStoreFiles();

    void finishedRotatingLogicalLogs();

    void streamedFile( File file );

    void streamingFile( File file );

    void recoveredStore();

    public static final StoreCopyMonitor NONE = new Adaptor();

    class Adaptor implements StoreCopyMonitor
    {
        @Override
        public void streamingFile( File file )
        {   // Do nothing
        }

        @Override
        public void recoveredStore()
        {   // Do nothing
        }

        @Override
        public void streamedFile( File file )
        {   // Do nothing
        }

        @Override
        public void startCopyingFiles()
        {   // Do nothing
        }

        @Override
        public void finishedRotatingLogicalLogs()
        {   // Do nothing
        }

        @Override
        public void finishedCopyingStoreFiles()
        {   // Do nothing
        }
    }
}
